@@ -2,6 +2,7 @@ package openapi3filter
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -15,6 +16,7 @@ import (
 	"testing"
 
 	"github.com/getkin/kin-openapi/openapi3"
+	legacyrouter "github.com/getkin/kin-openapi/routers/legacy"
 	"github.com/stretchr/testify/require"
 )
 
@@ -903,38 +905,29 @@ func TestDecodeParameter(t *testing.T) {
 						req.AddCookie(&http.Cookie{Name: v[0], Value: v[1]})
 					}
 
-					var path string
+					path := "/test"
 					if tc.path != "" {
-						switch tc.param.Style {
-						case "label":
-							path = "." + tc.param.Name
-						case "matrix":
-							path = ";" + tc.param.Name
-						default:
-							path = tc.param.Name
-						}
-						if tc.param.Explode != nil && *tc.param.Explode {
-							path += "*"
-						}
-						path = "/{" + path + "}"
+						path += "/{" + tc.param.Name + "}"
 					}
 
 					info := &openapi3.Info{
 						Title:   "MyAPI",
 						Version: "0.1",
 					}
-					spec := &openapi3.Swagger{OpenAPI: "3.0.0", Info: info}
+					spec := &openapi3.T{OpenAPI: "3.0.0", Info: info}
 					op := &openapi3.Operation{
 						OperationID: "test",
 						Parameters:  []*openapi3.ParameterRef{{Value: tc.param}},
 						Responses:   openapi3.NewResponses(),
 					}
-					spec.AddOperation("/test"+path, http.MethodGet, op)
-					router := NewRouter()
-					require.NoError(t, router.AddSwagger(spec), "failed to create a router")
+					spec.AddOperation(path, http.MethodGet, op)
+					err = spec.Validate(context.Background())
+					require.NoError(t, err)
+					router, err := legacyrouter.NewRouter(spec)
+					require.NoError(t, err)
 
-					route, pathParams, err := router.FindRoute(req.Method, req.URL)
-					require.NoError(t, err, "failed to find a route")
+					route, pathParams, err := router.FindRoute(req)
+					require.NoError(t, err)
 
 					input := &RequestValidationInput{Request: req, PathParams: pathParams, Route: route}
 					got, err := decodeStyledParameter(tc.param, input)
@@ -1015,7 +1008,7 @@ func TestDecodeBody(t *testing.T) {
 		wantErr  error
 	}{
 		{
-			name:    "unsupported content type",
+			name:    prefixUnsupportedCT,
 			mime:    "application/xml",
 			wantErr: &ParseError{Kind: KindUnsupportedFormat},
 		},
@@ -1134,7 +1127,7 @@ func TestDecodeBody(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
 			h := make(http.Header)
-			h.Set(http.CanonicalHeaderKey("Content-Type"), tc.mime)
+			h.Set(headerCT, tc.mime)
 			var schemaRef *openapi3.SchemaRef
 			if tc.schema != nil {
 				schemaRef = tc.schema.NewRef()
@@ -1180,7 +1173,7 @@ func newTestMultipartForm(parts []*testFormPart) (io.Reader, string, error) {
 		}
 
 		h := make(textproto.MIMEHeader)
-		h.Set("Content-Type", p.contentType)
+		h.Set(headerCT, p.contentType)
 		h.Set("Content-Disposition", disp)
 		pw, err := w.CreatePart(h)
 		if err != nil {
@@ -1194,39 +1187,42 @@ func newTestMultipartForm(parts []*testFormPart) (io.Reader, string, error) {
 }
 
 func TestRegisterAndUnregisterBodyDecoder(t *testing.T) {
-	var (
-		contentType = "text/csv"
-		decoder     = func(body io.Reader, h http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
-			data, err := ioutil.ReadAll(body)
-			if err != nil {
-				return nil, err
-			}
-			var vv []interface{}
-			for _, v := range strings.Split(string(data), ",") {
-				vv = append(vv, v)
-			}
-			return vv, nil
+	var decoder BodyDecoder
+	decoder = func(body io.Reader, h http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (decoded interface{}, err error) {
+		var data []byte
+		if data, err = ioutil.ReadAll(body); err != nil {
+			return
 		}
-		schema  = openapi3.NewArraySchema().WithItems(openapi3.NewStringSchema()).NewRef()
-		encFn   = func(string) *openapi3.Encoding { return nil }
-		body    = strings.NewReader("foo,bar")
-		want    = []interface{}{"foo", "bar"}
-		wantErr = &ParseError{Kind: KindUnsupportedFormat}
-	)
+		return strings.Split(string(data), ","), nil
+	}
+	contentType := "text/csv"
 	h := make(http.Header)
-	h.Set(http.CanonicalHeaderKey("Content-Type"), contentType)
+	h.Set(headerCT, contentType)
+
+	originalDecoder := RegisteredBodyDecoder(contentType)
+	require.Nil(t, originalDecoder)
 
 	RegisterBodyDecoder(contentType, decoder)
+	require.Equal(t, fmt.Sprintf("%v", decoder), fmt.Sprintf("%v", RegisteredBodyDecoder(contentType)))
+
+	body := strings.NewReader("foo,bar")
+	schema := openapi3.NewArraySchema().WithItems(openapi3.NewStringSchema()).NewRef()
+	encFn := func(string) *openapi3.Encoding { return nil }
 	got, err := decodeBody(body, h, schema, encFn)
 
 	require.NoError(t, err)
-	require.Truef(t, reflect.DeepEqual(got, want), "got %v, want %v", got, want)
+	require.Equal(t, []string{"foo", "bar"}, got)
 
 	UnregisterBodyDecoder(contentType)
-	_, err = decodeBody(body, h, schema, encFn)
 
-	require.Error(t, err)
-	require.Truef(t, matchParseError(err, wantErr), "got error:\n%v\nwant error:\n%v", err, wantErr)
+	originalDecoder = RegisteredBodyDecoder(contentType)
+	require.Nil(t, originalDecoder)
+
+	_, err = decodeBody(body, h, schema, encFn)
+	require.Equal(t, &ParseError{
+		Kind:   KindUnsupportedFormat,
+		Reason: prefixUnsupportedCT + ` "text/csv"`,
+	}, err)
 }
 
 func matchParseError(got, want error) bool {
